#include	"codes.h"
#include	"always.h"
#include	"delay.h"

/*

rule of thumb for ringbuffers:

1)	#define MIN and MAX
2)	declared as ringbuffer[MAX+1]
3)	any loops involving buffers should count from MIN right up to and including MAX
		ie: for (i=MIN;i<=MAX;i++)
4)	*any* increment in a ring buffer should be accompanied by a test for rollover
		ie: ringbuffer_index++;if (ringbuffer_index>MAX) ringbuffer_index=MIN;
5)	*any* decrement in a ring buffer should be accompanied by a test for rollover
		ie: ringbuffer_index--;if (ringbuffer_index<MIN) ringbuffer_index=MAX;
		
*/

//ring buffer containing data received from USART

#define SERIAL_IN_MAX 88

volatile bank2 unsigned char serial_in[SERIAL_IN_MAX+1];

//use to wait for ever until data is available in serial_in ring buffer, ie: while(serial_in_empty);
#define serial_in_empty		(serial_in_index_in==serial_in_index_out)

//use to check if data is available in serial_in ring buffer, ie: if (serial_in_full) {//do stuff};
#define serial_in_full		(serial_in_index_in!=serial_in_index_out)

//for putting data in
volatile unsigned char serial_in_index_in=0;

//for pulling data out
volatile unsigned char serial_in_index_out=0;

//for resetting ring buffer so its 'empty'
void serial_in_reset(void)
{
	serial_in_index_in=0;
	serial_in_index_out=0;
}

//returns latest character from USART
void serial_in_put(unsigned char data)
{
	serial_in[serial_in_index_in]=data;
	serial_in_index_in++;
	if (serial_in_index_in>SERIAL_IN_MAX)
	{
		serial_in_index_in=0;
	}
	//if ring buffer full, stop accepting characters
	if (serial_in_index_in==serial_in_index_out)
	{
		err(0);
		serial_in_index_in--;
	}
}

//puts latest character from USART into buffer
unsigned char serial_in_get(void)
{
	unsigned char data;
	
	if (serial_in_index_in!=serial_in_index_out)
	{
		//pull data out
		data=serial_in[serial_in_index_out];
		serial_in_index_out++;
		if (serial_in_index_out>SERIAL_IN_MAX)
		{
			serial_in_index_out=0;
		}
	}
	else
	{
		//tried to pull stuff out but nothing there
		err(1);
	}
	return data;
}

//returns the number of chars taken from USART waiting in the buffer
//if serial_in_empty==TRUE, serial_in_count()=0
//if 1 char waiting, serial_in_count()=1
//ie: if (serial_in_count()>=24) {//do stuff}
unsigned char serial_in_count(void)
{
	if (serial_in_index_in>=serial_in_index_out)
	{
		return (serial_in_index_in-serial_in_index_out);
	}
	else
	{
		return (((SERIAL_IN_MAX+1)-serial_in_index_out)+serial_in_index_in);
	}
}

//*****
//ring buffer containing data to be sent out of USART

#define SERIAL_OUT_MAX 70

volatile bank1 unsigned char serial_out[SERIAL_OUT_MAX+1];

//use to wait for ever until data is available in serial_out ring buffer, ie: while(serial_out_empty);
#define serial_out_empty		(serial_out_index_in==serial_out_index_out)

//use to check if data is available in serial_out ring buffer, ie: if (serial_in_full) {//do stuff};
#define serial_out_full		(serial_out_index_in!=serial_out_index_out)

//for putting data in
volatile unsigned char serial_out_index_in=0;

//for pulling data out
volatile unsigned char serial_out_index_out=0;

//for resetting ring buffer so its 'empty'
void serial_out_reset(void)
{
	serial_out_index_in=0;
	serial_out_index_out=0;
}

void serial_out_put(unsigned char data)
{
	serial_out[serial_out_index_in]=data;
	serial_out_index_in++;
	if (serial_out_index_in>SERIAL_OUT_MAX)
	{
		serial_out_index_in=0;
	}
	//if ring buffer full, stop accepting characters
	if (serial_out_index_in==serial_out_index_out)
	{
		err(2);
		serial_out_index_in--;
	}
}

unsigned char serial_out_get(void)
{
	unsigned char data;
	
	if (serial_out_index_in!=serial_out_index_out)
	{
		//pull data out
		data=serial_out[serial_out_index_out];
		serial_out_index_out++;
		if (serial_out_index_out>SERIAL_OUT_MAX)
		{
			serial_out_index_out=0;
		}
	}
	else
	{
		//tried to pull stuff out but nothing there
		err(3);
	}
	return data;
}

//*****
//test suite

/*

//to test serial_in_count()

serial_in_reset();
while(1)
{
	serial_in_put('A');
	if (serial_in_count()!=1)
	{
		b;
	}
	serial_in_get();
}
b;

*/

